% The params structure is used to store global parameters which will be
% used across the program.

%% Parameters to be set by the user

imageDir = fullfile('Images', '20.10.2017');
params.imageDir = imageDir;
params.imgFormat = 'cr2';               % jpg, cr2, nef... should match the output format of the camera
params.projResolution = [1080, 1920];   % projector resolution in pixels [vertical, horizontal]
params.projPatternSize = [1080, 1920];  % size of the projected pattern in pixels [vertical, horizontal]
params.numImagesToCapture = 10;         % number of iterations that the algorithm will execute
params.onlyGeometric = 0;               % performs only geometric calibration if set to 1
params.offlineMode = 0;                 % offlineMode option, to execute on already captured images
params.showFigures = 0;                 % if this mode is activated figures will be displayed to the user

% Printed pattern
params.printedPatternSize = [10, 15];   % number of squares in the printed pattern [vertical, horizontal]
params.squareSize = 10;                 % in mm
params.printedPatternColor = 'y';       % color of the printed calibration pattern ['y' or 'c']
params.isfirstSquareBlack = 1;          % is the top-left corner of the printed pattern black (inked) or white

%% Init directories

params.currentImageNum = 1;

% Image directory
if ~exist(imageDir, 'dir'), mkdir(imageDir); end

% Data directory
dataDir  = fullfile(imageDir, 'Data');
if ~exist(dataDir, 'dir'), mkdir(dataDir); end
params.dataDir = dataDir;

if params.offlineMode
    params.numIterations = numel(dir(fullfile(imageDir, ['*phase_1*.', params.imgFormat])));
end

%% Printed pattern

printedPoints = generateCheckerboardPoints(params.printedPatternSize, params.squareSize); % the corner coordinates start from [0, 0]
params.printedPoints = printedPoints + params.squareSize; % add an offset of one printed square

% Save the 3D corner coordinates
printedPoints = params.printedPoints;
save(fullfile(dataDir, 'X_printed_pattern'), 'printedPoints');

%% Projected pattern

% Create the projected checkerboard
switch params.printedPatternColor
    case {'c', 'C'}
        params.printedPatternColorChannel   = 1;
        params.projectedPatternColorChannel = 3;
    case {'y', 'Y'}
        params.printedPatternColorChannel   = 3;
        params.projectedPatternColorChannel = 1;
end

[patternFilename, projectedPoints, params] = generateCustomCheckerboard(params);
params.projectedPattern = imread(patternFilename);
params.projectedPoints = projectedPoints;

% Save the 2D corner coordinates of the projected pattern
save(fullfile(dataDir, 'x_projected_pattern'), 'projectedPoints');

%% Radiometric parameters
if ~params.onlyGeometric
    
    % Prepare canvases (meshgrid) for performing the prewarp and also
    % undoing the prewarp of the projected patterns
    ur = 1:params.projResolution(2);
    vr = 1:params.projResolution(1);
    [u, v] = meshgrid(ur, vr);
    params.u = u; params.v = v;
    warpedProjectionPattern   = zeros(params.projResolution(1), params.projResolution(2), 3);
    unwarpedProjectionPattern = zeros(params.projResolution(1), params.projResolution(2), 3);
    
    % Radiometric calibration parameters
    params.effectiveSurfaceRadioCalib = 0.25; % range: [0, 0.5]
    params.offsetFromCornerCenterRadiometric = 10; % in px
    params.projectorPointsRounded = round(params.projectedPoints);
    params.squareCropSize = round(min(params.projPatternSize(1) / params.printedPatternSize(1),...
                                      params.projPatternSize(2) / params.printedPatternSize(2)) * params.effectiveSurfaceRadioCalib);
    params.radiometricValues = [];
end

%% Miscellaneous

% Search window half-size for guided corner detection in step 2
params.wintx = 25; % in px
params.winty = 25; % in px

% Create a red image as a warning for automatic corner detection fail
params.redImage = cat(3,  ones(params.projResolution(1), params.projResolution(2), 'uint8'),...
                         zeros(params.projResolution(1), params.projResolution(2), 'uint8'),...
                         zeros(params.projResolution(1), params.projResolution(2), 'uint8')) * 255;

% Create a green image to signify successful calibration
params.greenImage = cat(3, zeros(params.projResolution(1), params.projResolution(2), 'uint8'),...
                            ones(params.projResolution(1), params.projResolution(2), 'uint8'),...
                           zeros(params.projResolution(1), params.projResolution(2), 'uint8')) * 255;
