%
% Code for our Simultaneous Geometric and Radiometric Calibration of a Projector-Camera Pair.
% It is built on top of the method described in the following paper:
%   Marjan Shahpaski, Luis Ricardo Sapaico, Gaspard Chevassus, and Sabine Susstrunk
%   "Simultaneous Geometric and Radiometric Calibration of a Projector-Camera Pair."
%   In IEEE Conference on Computer Vision and Pattern Recognition (CVPR), 2017.
%
% Marjan Shahpaski (marjan.shahpaski@epfl.ch)
% EPFL, Lausanne, 27.10.2017
% 
%% Main script
close all;
clc;

if ~exist('params', 'var')
    startup
else
    initParams;
end

% Show the original projector pattern in projectImage
if ~params.offlineMode
    projectImage(params.projectedPattern);
    pause(5);
end

% Run this command only once (it is required by the gPhoto library)
if ~params.offlineMode
    unix('killall PTPCamera');
end

while params.currentImageNum <= params.numImagesToCapture
    
    fprintf('\nImage number %d / %d\n', params.currentImageNum, params.numImagesToCapture);
    
    %% ////////////////////////////////////////////////////////////////////
    % /////////// STEP 1: Project the original projector pattern //////////
    % /////////////////////////////////////////////////////////////////////
    
    params.step = 1;
    
    % Acquire an image of the printed and projected patterns
    if ~params.offlineMode
        % Capture an image
        currentImg = capturePhoto(params);
    else
        % Load an already captured image
        currentImg = imread(fullfile(params.imageDir, [num2str(params.currentImageNum, '%.2d'), '_step_', num2str(params.step), '.', params.imgFormat]));
    end
    
    % Extract the corners of the printed pattern from the captured image
    [imagePointsPrinted, isCapturedImgGood] = extractCornersProcedure(params, currentImg, 0);
    if ~isCapturedImgGood; continue; end;
    
    % Extract the corners of the projected pattern from the captured image
    [imagePointsProjected, isCapturedImgGood] = extractCornersProcedure(params,currentImg, 1);
    if ~isCapturedImgGood; continue; end;
    
    % Extract the corners of the projected pattern from the projected image
    imagePoints = params.projectedPoints;
    save(fullfile(params.dataDir, [num2str(params.currentImageNum, '%.2d'), '_x_proj_step_1']), 'imagePoints');
    
    if ~params.offlineMode
        % Compute the locations of the printed corners in the projector image
        boardPointsProjImg = printedCornersProjImage(params, imagePointsPrinted, imagePointsProjected);
        
        % Highlight detected corners
        highlightDetectedCorners(params.projectedPattern, boardPointsProjImg, params.projectedPoints);
    end
    
    %% ////////////////////////////////////////////////////////////////////
    % ////////// STEP 2: Project the warped projector pattern /////////////
    % /////////////////////////////////////////////////////////////////////
    
    if ~params.onlyGeometric

        params.step = 2;
        
        % Compute the warped projector pattern
        [warpedProjectionPattern, H_projectorToBoard_prewarp] = prewarpProjectedPattern(params, imagePointsPrinted, imagePointsProjected);
        
        % Project the prewarped projector pattern
        if ~params.offlineMode
            projectImage(im2uint8(warpedProjectionPattern));
            pause(2);
            
            % Acquire an image of the printed and projected patterns
            currentImg = capturePhoto(params);
        else
            % Load an already captured image
            currentImg = imread(fullfile(params.imageDir, [num2str(params.currentImageNum, '%.2d'), '_step_', num2str(params.step), '.', params.imgFormat]));
        end
        
        %  Extract the corners of the printed pattern from the captured image
        [imagePointsPrinted, isCapturedImgGood] = extractCornersProcedure(params, currentImg, 2);
        if ~isCapturedImgGood; continue; end;
        
        % Extract the corners of the projected pattern from the captured image
        imagePointsProjected = computeCornersProjectedPatternByOffset(imagePointsPrinted, currentImg, params);
        
        % Save the corner coordinates of the projected pattern
        projectedPoints = computeProjectedCornersByHomograpy(params, H_projectorToBoard_prewarp);
        
        % Highlight detected corners
%         if ~params.offlineMode
%             highlightDetectedCorners(params.projectedPattern, boardPointsProjImg, projectedPoints);
%         end
        
        % Display the unwarped image
        if params.showFigures
            % Compute the homography that relates the projector image with the camera image
            H_projectorToCamera = compute_homography(imagePointsProjected', params.projectedPoints');
            
            % Unwarp the captured image
            unwarpedProjectionPattern = warpImage(params, currentImg, H_projectorToCamera, 0);
            
            % Display the unwarped image
            figure, imshow(unwarpedProjectionPattern);
            pause(0.3);
        end
    end
    
    %% Prepare for the next iteration
    
    if ~params.offlineMode
        % Show the original projector pattern in projectImage
        projectImage(params.projectedPattern);
        
        % Pause to give the user time to change the board orientation
        pause(3);
    end
    
    % Update the current image number
    params.currentImageNum = params.currentImageNum + 1;
end

% Show feedback to user that all images have been captured
if ~params.offlineMode
    projectImage(params.greenImage);
end

%% Perform the geometric and radiometric calibration

% Geometric calibration of the camera
cameraParams = calibCamera(params, currentImg);

% Geometric calibration of the projector
calibProjector(params, cameraParams, params.projectedPoints);

%  Radiometric calibration of the projector
if ~params.onlyGeometric
    % Extract the data for radiometric calibration
    computeAllRadiometricValues(params);
    
    % Perform radiometric calibration of the projector
    gamma = computeGamma(params);
end

% Show the original projector pattern in projectImage
if ~params.offlineMode
    projectImage(params.projectedPattern);
end
