function computeAllRadiometricValues(params)
% Computes radiometric values from the images captured in step 2.
% The function first unwarps the projected image, such that its corners
% match the corners of the original projected pattern. Then extracts and
% saves the radiometric values.

fnames = dir(fullfile(params.imageDir, ['*_step_2.', params.imgFormat]));
numImgs = params.numImagesToCapture;
numInternalCorners = prod(params.printedPatternSize - 1);
radiometricValues = zeros(numInternalCorners, 4, numImgs);

% Extract the radiometric values
fprintf('Computing radiometric values\n');
fprintf(['[', repmat('.', [1, numImgs]), ']\n[']);
for i = 1:numImgs
    % Update the progress bar
    fprintf('|');
    
    % Load the current image
    if params.imgFormat == 'jpg'
        % Load the jpg image and linearize it (assuming sRGB)
        currentImg = inversegamma(fullfile(params.imageDir, fnames(i).name));
    else
        try
            filepathRAW = fullfile(params.imageDir, fnames(i).name);
            [~, name] = fileparts(fnames(i).name);
            filepathTIFF = fullfile(params.imageDir, [name, '.tiff']);
            
            % Convert the RAW image to TIFF by using dcraw
            if ~exist(filepathTIFF, 'file')
                system(['dcraw -D -4 -T "', filepathRAW, '"']);
            end
            
            img = imread(filepathTIFF);
            currentImg = img(6:end-5, 10:end-9);
            currentImg = demosaic(currentImg, 'bggr');
        catch
            % Load the RAW image normally, and linearize it (assuming sRGB)
            currentImg = inversegamma(fullfile(params.imageDir, fnames(i).name));
        end
    end
    
    % Load the projected corners for the current image
    load(fullfile(params.dataDir, [num2str(i, '%.2d'), '_x_cam_projected_step_2'])); % contains 'imagePoints'
    projectedCorners = imagePoints;
    
    % Compute the homography that relates the projector image with the camera image (by using the projected corners)
    H_ProjectorToCamera = compute_homography(projectedCorners', params.projectedPoints');
    
    % Extract the average values of the subsquares for radiometric calibration
    unwarpedProjectionPattern = warpImage(params, currentImg, H_ProjectorToCamera, 0);
    
    % Display the unwarped image
    if params.showFigures
        figure, imshow(unwarpedProjectionPattern * 16); hold on; plot(params.projectedPoints(:, 1), params.projectedPoints(:, 2), 'g.', 'MarkerSize', 24);
        pause(0.3);
    end
    
    % Compute the average values around the projected checkerboard corners
    radiometricValues(:, :, i) = computeAverageRadiometricvalues(params, unwarpedProjectionPattern);
end
fprintf(']\n\n');

% Save the radiometric values to disk
save(fullfile(params.dataDir, 'radiometric_values'), 'radiometricValues');

end