function meanGamma = computeGamma(params)
% This function fits a power function into the extracted radiometric
% values. The images are processed independently, and gamma is estimated
% per image. Finally, output projector gamma value is the mean of the gamma
% values of all images.

fprintf('Computing gamma...\n');

[inVals, validIdx] = getIndexOfProjectedSquares(params);

% Load the extracted image intensities (output values)
load(fullfile(params.dataDir, 'radiometric_values.mat')); % contains 'radiometricValues'

% Fit a power function into the measured data
numImgs = params.numImagesToCapture;
projectorGamma = zeros(numImgs, 1);
fittingValues = zeros(numImgs, 3);
inValsAll = zeros(length(validIdx), 4, numImgs);
outValsAll = zeros(length(validIdx), 4, numImgs);
if params.showFigures, figure; end
for i = 1:numImgs
    % Indexing legend: (square corner index, 4 surrounding squares, image number)
    outVals = radiometricValues(validIdx, :, i); %#ok<*NODEF>
    
    % Fit a power function into the image data
    fitobject = fit(inVals(:), outVals(:), 'power2');
    projectorGamma(i) = fitobject.b;
    fittingValues(i, :) = [fitobject.c, fitobject.a, fitobject.b];
    
    % Plot the measured against the projected values
    if params.showFigures
        plot(inVals, outVals, 'b.'); hold on; % plot the data points
        x = linspace(0, 1, 11);
        plot(x, fitobject.c + fitobject.a * x .^ fitobject.b); % plot the fitted function
        title(['Image ', num2str(i), ': gamma = ', num2str(fitobject.b, '%.2f')]);
        pause(0.3);
    end
    
    % Assemble the data of all captured images
    inValsAll(:, :, i) = inVals;
    outValsAll(:, :, i) = outVals;
end

% Compute the mean gamma from the gammas estimated in each image
meanGamma = mean(projectorGamma);
if params.showFigures
    figure, plot(inValsAll(:), outValsAll(:), 'b.'); hold on
    plot(x, mean(fittingValues(:, 1)) + mean(fittingValues(:, 2)) * x .^ mean(fittingValues(:, 3)), 'r', 'LineWidth', 3);
    title(['Mean gamma = ', num2str(meanGamma, '%.2f')]);
    pause(0.3);
end

% Save the computed values to disk
save(fullfile(params.dataDir, 'projectorGamma'), 'projectorGamma', 'meanGamma', 'inValsAll', 'outValsAll');

% Display the computed gamma
fprintf('Projector gamma is:\n\n');
disp(meanGamma);

end