function [patternFilename, projectedPoints, params] = generateCustomCheckerboard(params)
% Generates the custom projection calibration chart, which allows for
% simultaneous geometric and radiometric calibration of the projector. When
% the parameter 'onlyGeometric' is set, it generates a conventional
% calibration chart used for geometric calibration only. This option can be
% used for generating the printed chart.

% Init parameters
numRows             = params.printedPatternSize(1);
numCols             = params.printedPatternSize(2);
projHeight          = params.projResolution(1);
projWidth           = params.projResolution(2);
projPatternHeight   = params.projPatternSize(1);
projPatternWidth    = params.projPatternSize(2);
printedColor        = params.printedPatternColor;
onlyGeometric       = params.onlyGeometric;

% Check if the number of columns is >= 10 for radiometric calibration
if numCols < 10 && ~onlyGeometric
    error('The checkerboard must have at least 10 columns for radiometric calibration.');
end

% Assemble the filename of the projector calibration chart
switch printedColor
    case {'c', 'C'}
        patternFilename = fullfile(params.imageDir, ['projector_calibration_chart-', num2str(numCols), 'x',...
            num2str(numRows), '_red_mag_', num2str(projPatternWidth), 'x', num2str(projPatternHeight), '.png']);
    case {'y', 'Y'}
        patternFilename = fullfile(params.imageDir, ['projector_calibration_chart-', num2str(numCols), 'x',...
            num2str(numRows), '_blue_mag_', num2str(projPatternWidth), 'x', num2str(projPatternHeight), '.png']);
end

if ~onlyGeometric
    patternFilename = [patternFilename(1:end-4), '_radio.png'];
end

% Compute the size of the squares
squareSizeX = ceil(projPatternWidth  / numCols); % in px
squareSizeY = ceil(projPatternHeight / numRows); % in px

% Build the gradient for geometric and radiometric calibration
if onlyGeometric
    checkerboardPattern = zeros(numRows, numCols);
    checkerboardPattern(2:2:end, 1:2:end) = 1;
    checkerboardPattern(1:2:end, 2:2:end) = 1;
else
    midCol = floor(numCols / 2);
    checkerboardTile = [0, 6, 2, 8, 4, 10, 4, 8, 2, 6;
                        5, 1, 7, 3, 9,  5  9, 3, 7, 1] / 10;
    
    s = size(checkerboardTile);
    checkerboardFull = repmat(checkerboardTile, [ceil(numRows / s(1)), ceil(numCols / s(2))]);
    checkerboardPattern = circshift(checkerboardFull, [0, midCol]); % shift zero intensity to the center
    checkerboardPattern = checkerboardPattern(1:numRows, 1:numCols);
    params.checkerboardPattern = checkerboardPattern;
end

% Convert the checkerboard dots into squares
checkerboardPatternProjSize = kron(checkerboardPattern, ones(squareSizeY, squareSizeX));

% Add color to the checkerboard
[checkerboardHeight, checkerboardWidth, ~] = size(checkerboardPatternProjSize);
checkerboardPatternProjSizeColor = zeros(checkerboardHeight, checkerboardWidth, 3);
checkerboardPatternProjSizeColor(:, :, params.printedPatternColorChannel) = 1;
checkerboardPatternProjSizeColor(:, :, params.projectedPatternColorChannel) = checkerboardPatternProjSize;

% Fit the checkerboard pattern into the projector resolution
checkerboardPatternProjSizeBack = ones(max(projHeight, checkerboardHeight), max(projWidth, checkerboardWidth), 3);
checkerboardPatternProjSizeBack(:, :, 2) = 0; % make the background magenta
checkerboardPatternProjSizeBack(1:checkerboardHeight, 1:checkerboardWidth, :) = checkerboardPatternProjSizeColor;

if projHeight > projPatternHeight % for wide angle projectors, shift the checkerboard to the bottom
    shiftRows = projHeight - checkerboardHeight;
else
    shiftRows = round((projHeight - checkerboardHeight) / 2);
end
shiftCols = round((projWidth - checkerboardWidth) / 2);

checkerboardPatternProjSizeBack = circshift(checkerboardPatternProjSizeBack, [shiftRows, shiftCols]);
checkerboardPatternProjSizeFinal = checkerboardPatternProjSizeBack(1:projHeight, 1:projWidth, :);

% Calculate the corner positions
cornerPositionsX = squareSizeX+0.5:squareSizeX:checkerboardWidth;
cornerPositionsY = squareSizeY+0.5:squareSizeY:checkerboardHeight;
cornerPositionsX = cornerPositionsX + shiftCols;
cornerPositionsY = cornerPositionsY + shiftRows;
[X, Y] = meshgrid(cornerPositionsX, cornerPositionsY);
projectedPoints = [X(:), Y(:)];

% Display the checkerboard
if params.showFigures, figure, imshow(checkerboardPatternProjSizeFinal); pause(0.3); end

% Save the checkerboard to the disk
imwrite(checkerboardPatternProjSizeFinal, patternFilename);

end